@@ -1,5 +1,5 @@
 
 .onLoad <- function(libname=NULL, pkgname=feature)
 {
-  cat("feature 1.1-9 (2007)\n") 
+  #cat("feature 1.1-9 (2007)\n") 
 }
